--[[----------------------------------------------------------------------------------------------------------------------------
-- This Rift Addon will show a Killing Blow message in a frame when the Player kills a target.
-- You can enable a message for when the Player gets killed by another Unit.
-- If enabled, will count a killing streak which resets on death or zone change.
-- @Author: Aurolieen@EU-BLOODIRON
-- @LastUpdate: 02-11-2013
-- @Version: 1.2
-- @Credits: Adelea@EU-Icewatch - For the ZoneChange library.
--------------------------------------------------------------------------------------------------------------------------------
-- Changelog:
--		1.0: Initial Release
-- 		1.1: Added killing streak functionality.
--		1.2: Fixed several bugs due to deprecated functionalities.
----------------------------------------------------------------------------------------------------------------------------]]--
Killingblow = {
	version = "1.1",
	messages = {},
	lastUpdate = 0,
	currentStreak = 0,
	oldLocation = "",
	config = {
		ttl = 4,				-- The time in seconds a message should be staying on the screen.
		showPlayerDeath = true,	-- Whether or not the addon should display a message when the player dies.
		showStreak = false,		-- Whether or not the addon should display your killingblow streak.
		size = 32,				-- Font size of the messages.
		colour = {				-- Colour of the messages. Values between 0 and 1.
			red = 1,
			green = 0,
			blue = 0,
			alpha = 0			-- Transparancy, 0 is opaque, 1 is fully transparant.
		}
	}
}

-- Set up the Text Frame.
Killingblow.context = UI.CreateContext("Killingblow_Context");
Killingblow.frame = UI.CreateFrame("Text", "Killingblow", Killingblow.context);
Killingblow.frame:SetFontSize(Killingblow.config.size);
Killingblow.frame:SetFontColor(Killingblow.config.colour.red, Killingblow.config.colour.green, 
								Killingblow.config.colour.blue, Killingblow.config.colour.alpha);
Killingblow.frame:SetText(Killingblow.version);

-- Position on the screen, should you want to change it.
Killingblow.frame:SetPoint("TOPCENTER", UIParent, "TOPCENTER", 0, 100);

-- OnEvent.Combat.Death
local function OnDeathEventHandler(pHandle, pTable)
	player = Inspect.Unit.Detail("player");
	caster = pTable.casterName;
	target = pTable.targetName;
	
	-- This fixes a rare bug in which the Event sends an incomplete table. (blame the devs)
	if caster == nil then
		caster = "Unknown Entity";
	end
	if target == nil then
		target = "Unknown Entity";
	end
	
	-- Check if the Player scored the kill.
	if caster == player["name"] then
		-- Add a Killing Blow message to the text frame.
		if Killingblow.config.showStreak then
			Killingblow.currentStreak = Killingblow.currentStreak + 1;
			table.insert(Killingblow.messages, 1, 
								{message = "KILLING BLOW: "..target.." #"..Killingblow.currentStreak,
									ttl = Killingblow.config.ttl});
		else
			table.insert(Killingblow.messages, 1, {message = "KILLING BLOW: "..target,
													ttl = Killingblow.config.ttl});
		end
	-- Otherwise check if the Player died.
	elseif ((target == player["name"]) and Killingblow.config.showPlayerDeath) then
		-- Reset the Killing Streak.
		if Killingblow.config.showStreak then
			Killingblow.currentStreak = 0;
		end
		-- Add a Death message to the text frame.
		table.insert(Killingblow.messages, 1, {message = "KILLED BY: "..caster, ttl = Killingblow.config.ttl});
	end
end

-- OnEvent.System.Update.Begin
local function OnUpdateEventHandler(pHandle)
	list = "";
	currentFrameTime = Inspect.Time.Frame();
	-- Execute every second of time.
	if (currentFrameTime - Killingblow.lastUpdate) >= 1 then
		-- Loop through the table of messages and remove all the ones where ttl-1 == 0.
		i = 1;
		while (i <= #Killingblow.messages) do
		-- Decrease the ttl of the message by 1.
			Killingblow.messages[i].ttl = Killingblow.messages[i].ttl - 1;
			if Killingblow.messages[i].ttl == 0 then
				table.remove(Killingblow.messages, i);
				-- 1 less result in the table so keep i the same.
			else
				i=i+1;
			end
		end
		-- Update the old time to the new time.
		Killingblow.lastUpdate = currentFrameTime;
	end
	-- Create the text for the frame from the messages array.
	i = 1;
	while (i <= #Killingblow.messages) do
		list = list..Killingblow.messages[i].message.."\n";
		i=i+1;
	end
	-- Update the frame text.
	Killingblow.frame:SetText(list);

end

-- OnEvent.Library.libZoneChange.Player
local function OnZoneChangeHandler(pHandle, pZoneName, pZoneID)
	-- Reset the killing streak to 0.
	Killingblow.currentStreak = 0;
end

--Set Rift Events.
Command.Event.Attach(Event.Combat.Death, OnDeathEventHandler, "OnDeathEventHandler");
Command.Event.Attach(Event.System.Update.Begin, OnUpdateEventHandler, "OnUpdateEventHandler");
-- Set the libZoneChange Event if showStreak is enabled.
if Killingblow.config.showStreak then
	Command.Event.Attach(Library.libZoneChange.Player, OnZoneChangeHandler, "Library.LibZoneChange.Player");
end